//------------------------------------------------------------------------
//
// Author      : Daniel Suess
// Email       : daniel.suess@web.de
// Website     : -------------------
// Date        : 16. Mrz 2002
// Version     : 1.0
// Description : Bumpmapping Sample for DGL
// Visit http://www.delphigl.com
//
//------------------------------------------------------------------------

unit Mainunit;

interface

uses
  Windows, ExtCtrls, Classes, AppEvnts, OpenGL12, Forms, Controls,
  SysUtils, Dialogs, glBmp, Messages, NVREGCOMB, Geometry;

  //                           Alte Uses:
  //Messages, Variants, Graphics;



const
  CCaption = 'OPENGl Delphi Test';
  NumTex = 2;           //Anzahle Texturen
  ObjList = 1;         //Displaylist des Objektes
type 
  T3DPoint = record
    x, y, z: double;
  end;

type
  TMainform = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    FPSTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: boolean);
    procedure FormResize(Sender: TObject);
    procedure FPSTimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: word;
      Shift: TShiftState);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: integer);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    FDC: HDC;
    FRC: HGLRC;
    Frames: integer;
    yrot: TGLFloat;
    xrot: TGLFloat;
    OldMPos: TPoint;
    Textures: array[0..NumTex - 1] of TGLBMP;
    keys: array[0..255] of boolean;
    Move_Pos: TGLUInt;

    LightAngle: TGLFloat;

    procedure GLInit;
    procedure LoadTextures;
    procedure DrawScene;
    procedure InitRoom;
  end;

var 
  started: boolean = False;
var 
  Mainform: TMainform;
var 
  ElapsedTime: word = 0;

implementation

{$R *.dfm}
procedure TMainform.InitRoom;
begin
  glNewList(ObjList, GL_COMPILE);
  glShadeModel(GL_SMOOTH);
  glDepthFunc(GL_LESS);

  {         Textur         }
  glActiveTextureARB(GL_TEXTURE1_ARB);
  glEnable(GL_TEXTURE_2D);
  Textures[1].Bind;

  {        Bumpmap      }
  glActiveTextureARB(GL_TEXTURE0_ARB);
  glEnable(GL_TEXTURE_2D);
  Textures[0].Bind;

  glEnable(GL_REGISTER_COMBINERS_NV);

  glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV, 1);

  glCombinerInputNV(GL_COMBINER0_NV, //Combiner, der benutzt wird
    GL_RGB,
    GL_VARIABLE_A_NV, //Variable beim Combiner
    GL_TEXTURE1_ARB, //Wert: Bumpmap Textur
    GL_EXPAND_NORMAL_NV,
    GL_RGB
    );

  glCombinerInputNV(GL_COMBINER0_NV, //Combiner, der benutzt wird, wie oben
    GL_RGB,
    GL_VARIABLE_B_NV, // Variable beim Combiner
    GL_PRIMARY_COLOR_NV, //Wert: RGB Wert der Oberflche
    GL_EXPAND_NORMAL_NV,
    GL_RGB
    );

  glCombinerOutputNV(GL_COMBINER0_NV,
    GL_RGB,
    GL_SPARE0_NV, // Output von A und B
    GL_DISCARD_NV, // Output von C und D
    GL_DISCARD_NV, // Summe von allen Variablen
    GL_NONE, // Skalierung
    GL_NONE, // Bias
    true, // Ja/Nein Output von A und B
    false, // Ja/Nein Output von C und D
    false // Muxsum
    );

  glFinalCombinerInputNV(GL_VARIABLE_A_NV, GL_SPARE0_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);

  glFinalCombinerInputNV(GL_VARIABLE_B_NV, GL_TEXTURE0_ARB, GL_UNSIGNED_IDENTITY_NV, GL_RGB);

  glFinalCombinerInputNV(GL_VARIABLE_C_NV, GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);

  glFinalCombinerInputNV(GL_VARIABLE_D_NV, GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);

  glFinalCombinerInputNV(GL_VARIABLE_E_NV, GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);

  glFinalCombinerInputNV(GL_VARIABLE_F_NV, GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);


  glBegin(GL_QUADS);
  {         Bottom        }
  //glColor3f(0.6, 0.6, 0.6);
  glNormal3f(0,1, 0);

  glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0, 0);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0, 0);
  glVertex3f(-5, 0, -5);

  glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0, 1);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 0, 1);
  glVertex3f(-5, 0, 5);

  glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1, 1);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1, 1);
  glVertex3f(5, 0, 5);

  glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1, 0);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB, 1, 0);
  glVertex3f(5, 0, -5);

  glEnd;
  glDisable(GL_TEXTURE_2D);
  glEndList;
end;

procedure TMainform.FormCreate(Sender: TObject);
var
  Dummy: HPalette;
begin
  WindowState := wsMaximized;
  BorderStyle := bsNone;
  //Cursor := crNone;
  FDC := GetDC(Handle);
  // Generate our rendering context with a color depth of 32Bpp
  FRC := CreateRenderingContext(FDC, [opDoubleBuffered], 32, 24, 0, 0, 0, 0, Dummy);
  Frames := 0;
  GLInit;

  LoadTextures;
  InitRoom;
  started := True;
end;

// =============================================================================
// Sets up all needed OpenGL properties
// =============================================================================
procedure TMainform.GLInit;
var
  TMUs: integer;
  MaxCombiners: integer;
begin
  ActivateRenderingContext(FDC, FRC);
  // Check, if the GFX-Card supports Multitexturing
  glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, @TMUs);
  if TMUs < 2 then
  begin
    ShowMessage('Multitexturing not supported');
    Close;
  end;
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glGetIntegerv(GL_MAX_GENERAL_COMBINERS_NV, @MaxCombiners);
  ReadExtensions;
  ReadNVREGCOMBExtension;
  if not (Assigned(@glCombinerParameteriNV) and Assigned(@glActiveTextureARB)) then
  begin
    ShowMessage('Register Combiner not supported');
    Close;
  end;
end;

// =============================================================================
// Loads all needed textures
// =============================================================================
procedure TMainform.LoadTextures;
const
  Name: array[0..NumTex - 1] of string = ('base', 'bump');
var
  i: integer;
begin
  for i := 0 to High(Name) do
  begin
    Textures[i] := TGLBmp.Create;
    Textures[i].LoadImage('textures\' + Name[i] + '.tga');
    Textures[i].GenTexture(False, False);
  end;
end;

// =============================================================================
// Draws the Scene
// =============================================================================
procedure TMainform.DrawScene;
  procedure InitLight;
  var
    light_position: TVertex;
  begin
    light_position[0] := cos(lightangle);
    light_position[2] := 1;
    light_position[1] := sin(lightangle);
    NormalizeVector(light_position);
    glColor3fv(@light_position[0]);
  end; (*InitLight*)
var

  Error : LongInt;
begin
  glMatrixMode(GL_MODELVIEW);
  glClearColor(0.3,0.3,0.3,0);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity;

  {------------------------------------------------------------------------------}
  {       Ab hier eigenes Zeugs                                                  }
  {------------------------------------------------------------------------------}

  glLoadIdentity;
  glMatrixMode(GL_MODELVIEW);
  gluLookAt(0,14,0, 0,0,0, 1, 0, 0);
  glRotatef(xrot, 1, 0, 0);
  glRotatef(yrot, 0, 1, 0);

  InitLight;
  LightAngle := LightAngle - 0.01;

  glEnable(GL_TEXTURE_2D);
  glCallList(ObjList);

  Error := glGetError;
  if Error <> GL_NO_ERROR then
  begin
    ShowMessage(gluErrorString(Error))
  end;
  {------------------------------------------------------------------------------}
  {       Ende des Zeichen Zeugs, ab hier nichts ndern                          }
  {------------------------------------------------------------------------------}
  SwapBuffers(FDC);
end;


procedure TMainform.ApplicationEvents1Idle(Sender: TObject;
  var Done: boolean);
begin
  if not Started then exit;
  DrawScene;
  Done := False;
  inc(Frames);
end;

procedure TMainform.FormResize(Sender: TObject);
begin
  if not Started then
    exit;
  if HandleAllocated then
  begin
    glViewport(0, 0, ClientWidth, ClientHeight);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(45, ClientWidth / ClientHeight, 0.1, 1000);
  end;
end;

procedure TMainform.FPSTimerTimer(Sender: TObject);
begin
  Caption := ccaption + ' - ' + IntToStr(Frames) + ' FPS';
  Frames := 0;
end;

procedure TMainform.FormKeyDown(Sender: TObject; var Key: word;
  Shift: TShiftState);
begin
  keys[Key] := True;
  if key = 27 then Close;   //Wenn ESC sind wir fertig
end;

procedure TMainform.FormKeyUp(Sender: TObject; var Key: word;
  Shift: TShiftState);
begin
  keys[key] := False;
end;

procedure TMainform.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: integer);
var 
  NewMPos: TPoint;
begin
  NewMPos := Mouse.CursorPos;
  if OldMPos.X < NewMPos.X then
    yrot := yrot + Abs(OldMPos.X - NewMPos.X) / 2;
  if OldMPos.X > NewMPos.X then
    yrot := yrot - Abs(OldMPos.X - NewMPos.X) / 2;
  if (OldMPos.Y > NewMPos.Y) {and (xrot>-60)} then
    xrot := xrot - Abs(OldMPos.Y - NewMPos.Y) / 4;
  if (OldMPos.Y < NewMPos.Y) {and (xrot < 60)} then
    xrot := xrot + Abs(OldMPos.Y - NewMPos.Y) / 4;
  OldMPos := NewMPos;
  if NewMPos.X < Left + 10 then SetCursorPos(Left + ClientWidth - 11, NewMPos.Y);
  if NewMPos.X >= Left + ClientWidth - 10 then SetCursorPos(Left + 11, NewMPos.Y);
  if NewMPos.Y < Top + 30 then SetCursorPos(NewMPos.X, Top + ClientHeight - 11);
  if NewMPos.Y >= Top + ClientHeight - 10 then SetCursorPos(NewMPos.X, Top + 31);
  NewMPos := Mouse.CursorPos;
  OldMPos := NewMPos;
end;

procedure TMainform.FormDestroy(Sender: TObject);
begin
  DeactivateRenderingContext();
  DestroyRenderingContext(FRC)
end;

end.
